
clear
addpath '../routines'
addpath '../examples'


% Abbruchtoleranz 
tol=1e-6;

bspstr{1}='eindimensional';
bspstr{2}='zweidimensional';
bspstr{3}='ENDE';

bsp=menu('Modellbeispiel',bspstr);
if bsp==3
   return
end
% Problemgre
if bsp==1
   N=200;
else 
   N=40;
end
T=spdiags(ones(N,1)*[-1 2 -1],-1:1,N,N);
h=1/(N+1);
lambda1=2*sin(1*pi*h/2); lambda1=lambda1*lambda1;
lambdan=2*sin(N*pi*h/2); lambdan=lambdan*lambdan;
lambdavec=2*sin((1:N)*pi*h/2); lambdavec=lambdavec.*lambdavec;

t=(0:h:1)';
x1=sin(1*pi*t); x1=x1./norm(x1,2);
xn=sin(N*pi*t); xn=xn./norm(xn,2);

if bsp==1
   n=N;
   A=T;
else
   n=N*N;
   A=kron(T,speye(N))+kron(speye(N),T);
   lambda1=2*lambda1;
   lambdan=2*lambdan;
   lambdavec=ones(N,1)*lambdavec+lambdavec'*ones(1,N);
end

ewn=3;
ewstr{1}='(betragl.) grter Eigenwert';
ewstr{2}='(betragl.) kleinster Eigenwert';
ewstr{3}='ENDE';

methodn=3;
methodstr{1}='Potenzmethode bzw. inverse Iteration';
methodstr{2}='Rayleigh-Quotienten-Iteration';
methodstr{3}='ENDE';


ew=menu('Eigenwert/Eigenvektorberechnung',ewstr);
fprintf('%s\n',ewstr{ew});

while ew<ewn
      if ew==1
	 methodstr{1}='Potenzmethode';
      else
	 methodstr{1}='Inverse Iteration';
      end % if-else

      method=menu('Verfahren',methodstr);
      fprintf('%s\n',methodstr{method});
   
      if method==methodn
	 return;
      end % if 
      
      x=randn(n,1)-0.5;
      x=x/norm(x);
      t0 = clock;
      if ew==1 & method==1
	 [x,lambda,k,res]=POWERMETHOD(A,x,tol);
      elseif ew==2 & method==1
	 [x,lambda,k,res]=INVERSEITERATION(A,x,tol);
      elseif ew==1 & method==2
	 lambda=norm(A,1);
	 [x,lambda,k,res]=RQI(A,x,lambda,tol);
      else % ew==2 & method==2
	 lambda=0;
	 [x,lambda,k,res]=RQI(A,x,lambda,tol);
      end % if-elseif-else
      t0=etime(clock,t0);
      
      % Whle passendes Vorzeichen verglichen mit der exakten Lsung
      if x(1)<0
	 x=-x;
      end
      
      
      figure(1)
      if bsp==1
	 plot(t,[0;x;0],'+r');
      else
	 X=zeros(N+2,N+2);
	 for i=1:N
	     X(2:N+1,i+1)=x((i-1)*N+1:i*N);
	 end
	 mesh(t,t,X);
      end
      title('berechneter Eigenvektor')

      fprintf('Zeit [sec]: %8.1e\n',t0);
      fprintf('Anzahl Iterationsschritte: %6d\n',k);
      if bsp==1
	 [lambdal,l]=min(abs(lambdavec-lambda));
	 lambdal=lambdavec(l);
	 fprintf('Approximierter Eigenwert lambda_%d=%16.8e\n',...
	         l,lambdal);
         m=l;
	 xl=sin(l*pi*t); xl=xl./norm(xl,2);
	 xm=xl;
      else
	 lambdal=inf;
	 for i=1:N
	     [ml,j]=min(abs(lambdavec(:,i)-lambda));
	     if ml<lambdal
		lambdal=ml;
		m=i;
		l=j;
	     end
	 end % for i
	 xl=sin(l*pi*t); xl=xl./norm(xl,2);
	 xm=sin(m*pi*t); xm=xm./norm(xm,2);
	 % doppelter Eigenwert, ermittle passende Linearkombination
	 if m~=l
	    tt=(h:h:(1-h))';
	    xll=sin(l*pi*tt); xll=xll./norm(xll,2);
	    xmm=sin(m*pi*tt); xmm=xmm./norm(xmm,2);
	    xml=kron(xll,xmm);
	    xlm=kron(xmm,xll);
	    nrm=inf;
	    for ttheta=0:.01:2*pi
	        ss=sin(ttheta);
		cc=cos(ttheta);
		if norm(cc*xml+ss*xlm-x)<nrm
		   nrm=norm(cc*xml+ss*xlm-x);
		   s=ss;
		   c=cc;
		   theta=ttheta;
		end
	    end
	    I=theta-.02:.0001:theta+.02; 
	    for ttheta=I
	        ss=sin(ttheta);
		cc=cos(ttheta);
		if norm(cc*xml+ss*xlm-x)<nrm
		   nrm=norm(cc*xml+ss*xlm-x);
		   s=ss;
		   c=cc;
		   theta=ttheta;
		end
	    end
	    xml=c*xml+s*xlm;
	    Xml=zeros(N+2,N+2);
	    for i=1:N
	       Xml(2:N+1,i+1)=xml((i-1)*N+1:i*N);
	    end
	 else
	    Xml=xm*xl';
	 end
	 lambdal=lambdavec(m,l);
	 fprintf('Approximierter Eigenwert lambda_(%d,%d)=%16.8e\n',...
	         l,m,lambdal);
      end
      
      fprintf('berechneter Eigenwert|nchster Eigenwert|relat. Fehler\n');
      fprintf('-----------------------------------------------------\n');
      fprintf('   %16.8e  | %16.8e |%12.4e\n',...
              lambda,lambdal,abs((lambda-lambdal)/lambdal));

      figure(2);
      if bsp==1
	 plot(t,xl,'+b');
	 fprintf('relativer Fehler Eigenvektoren %12.4e\n',...
	         norm([0;x;0]-xl,inf)/norm(xl,inf));
      else
	 mesh(t,t,Xml);
	 fprintf('relativer Fehler Eigenvektoren %12.4e\n',...
                 max(max(abs(X-Xml)))/max(max(abs(Xml))));
      end
      title('exakter nchster Eigenvektor')

      figure(3);
      t1=(0:0.001:1)';
      x11=sin(1*pi*t1); x11=x11*(max(abs(x1))/max(abs(x11)));
      xnn=sin(N*pi*t1); xnn=xnn*(max(abs(xn))/max(abs(xnn)));
      if ew==1
	 if bsp==1
	    fprintf('Gesuchter Eigenwert lambda_%d=%16.8e\n',n,lambdan);
	    plot(t1,xnn,':g');
	    hold on;
	    plot(t,xn,'+g');
	    hold off;
	 else
	    fprintf('Gesuchter Eigenwert lambda_(%d,%d)=%16.8e\n',N,N,lambdan);
	    mesh(t,t,xn*xn');
	 end
	 title('gesuchter exakter Eigenvektor (hochfrequente Sinusschwingung)')
      else	 
	 if bsp==1
	    plot(t1,x11,':g');
	    hold on;
	    plot(t,x1,'+g');
	    hold off;
	    fprintf('Gesuchter Eigenwert lambda_1=%16.8e\n',lambda1);
	 else
	    mesh(t,t,x1*x1');
	    fprintf('Gesuchter Eigenwert lambda_(1,1)=%16.8e\n',lambda1);
	 end
	 title('gesuchter exakter Eigenvektor (niedrigfrequente Sinusschwingung)')
      end
      fprintf('\n');

      figure(4)
      semilogy(1:k,res(1:k),':+r');
      xlabel('Iterationsschritte')
      ylabel('||Ax-\lambda x||/|\lambda|');
      title('relatives Eigenvektorresiduum');
      
      ew=menu('Eigenwert/Eigenvektorberechnung',ewstr);
      fprintf('%s\n',ewstr{ew});

end % while
